<?php

class SpecialIrodsFileDownload extends SpecialPage {

    function __construct() {
        $dir = dirname(__FILE__) . '/';
        require_once ($dir . "RodsWiki.functions.php");

        parent::__construct('SpecialIrodsFileDownload', '', false);
    }

    function execute($par) {

        global $configIrodsRootPath, $IrodsAccount;
        $this->setHeaders();
        require_once (dirname(__FILE__) . "/RodsWiki.functions.php");
        require_once (dirname(__FILE__) . "/RodsWiki.install.php");
        
        
        if (isset($_GET['ticket']))
            $ticket = $_GET['ticket'];
        else
            $ticket = '';

        $IrodsAccount = getAccount(false, $ticket);

        $load_file_from_local_server = false; // set false to load file from IRODS server; set to load the file directly from the local server


        if (isset($_GET['filename'])) {
            $file = strip_tags(trim($_GET['filename'])); // extract the tags and trim the input filename
            if (isset($_GET['exists'])) {
                global $configIrodsRootPath, $additional_path;
                if (existFile($configIrodsRootPath . $additional_path . get_wiki_file_path($_GET['filename'])))
                    echo '1';
                else
                    echo '0';
                exit;
            }
        } else {
            $load_file_from_local_server = true;
            $file = "";
        }


// Gather relevent info about file
        if (isset($_GET['conversion'])) {
            $file_conversion = strip_tags(trim($_GET['conversion'])); // extract the tags and trim the input filename
            $file_conversion = basename($file_conversion); // extract the name of the file, without preceding path
            $file_extension = strtolower(substr(strrchr($file_conversion, "."), 1)); // extract the extention
        }

        $filename = basename($file); // extract the name of the file, without preceding path
        $file_extension = strtolower(substr(strrchr($file, "."), 1)); // extract the extention
// This will set the Content-Type to the appropriate setting for the file
        switch ($file_extension) {
            case "pdf":
                $ctype = "application/pdf";
                break;
            case "exe":
                $ctype = "application/octet-stream";
                break;
            case "zip":
                $ctype = "application/zip";
                break;
            case "doc":
                $ctype = "application/msword";
                break;
            case "xls":
                $ctype = "application/vnd.ms-excel";
                break;
            case "ppt":
                $ctype = "application/vnd.ms-powerpoint";
                break;
            case "gif":
                $ctype = "image/gif";
                break;
            case "png":
                $ctype = "image/png";
                break;
            case "jpeg":
            case "jpg":
                $ctype = "image/jpg";
                break;
            case "mp3":
                $ctype = "audio/mpeg";
                break;
            case "wav":
                $ctype = "audio/x-wav";
                break;
            case "mpeg":
            case "mpg":
            case "mpe":
                $ctype = "video/mpeg";
                break;
            case "mov":
                $ctype = "video/quicktime";
                break;
            case "avi":
                $ctype = "video/x-msvideo";
                break;

            // The following are for extensions that shouldn't be downloaded (sensitive stuff, like php files)
            case "php":
            case "htm":
            case "html":
                die("<b>Cannot be used for " . $file_extension . " files!</b>");
                break;


            default:
                $ctype = "application/force-download";
        }


        if ($load_file_from_local_server == false) {

            // create a file object, specifing the path of the file to be read
            if (!isset($_GET['conversion']))
                if (substr($file, 0, 1) == "/")
                    $full_file_path = $file;
                else
                    $full_file_path = $configIrodsRootPath . get_wiki_file_path($filename);
            else
                $full_file_path = $configIrodsRootPath . get_wiki_file_path($filename, 0) . "conv-" . filename_to_legal_characters($filename) . '/' . $file_conversion;


            $myfile = new ProdsFile($IrodsAccount, $full_file_path);

            //check if the file exists
            if ($myfile->exists()) {

                try {

                    //Begin writing headers
                    header("Pragma: public");
                    header("Expires: 0");
                    header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
                    header("Cache-Control: public");
                    header("Content-Description: File Transfer");

                    //Use the switch-generated Content-Type
                    header("Content-Type: $ctype");

                    //Force the download
                    if (isset($_GET['conversion']))
                        $header = "Content-Disposition: attachment; filename=" . $file_conversion . ";";
                    else
                        $header = "Content-Disposition: attachment; filename=" . $filename . ";";
                    header($header);
                    header("Content-Transfer-Encoding: binary");
                    header('Last-Modified: ' . date('r'));


                    $myfile->open("r"); // open in secure mode
                    $fileinfo = $myfile->getStats();
                    header("Content-Length: " . $fileinfo->size);

                    $myfile->rewind(); // move the cursor to the begining of the file
                    
                    // streaming chunks
                    ob_clean();
                    while ($str = $myfile->read(10485760)) { // Variable is updated at every 1MB read from the server
                        print($str);
                        ob_flush();
                        flush();
                    }
                    ob_end_clean();

                    $myfile->close();
                } catch (Exception $e) {
                    if ($e->getCode() == -818000)
                        die("This iRODS account has no rights to read this file.");
                }
            } else { // in case the file was not found on the IRODS Server
                $load_file_from_local_server = true;
            }
        } else { // in case there is no connection to the IRODS Server
            $load_file_from_local_server = true;


            // in case the file was not loaded from the IRODS server, look on the local server
            if (file_exists("../../images/" . get_wiki_file_path($filename)) && isset($_GET['filename'])) { // check if the file exists on the local server
                //Begin writing headers
                header("Pragma: public");
                header("Expires: 0");
                header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
                header("Cache-Control: public");
                header("Content-Description: File Transfer");

                //Use the switch-generated Content-Type
                header("Content-Type: $ctype");

                //Force the download
                $header = "Content-Disposition: attachment; filename=" . $filename . ";";
                header($header);
                header("Content-Transfer-Encoding: binary");
                header("Content-Length: ".filesize($file));
                header('Last-Modified: ' . date('r'));

                ob_clean();
                flush();

                // The PDF source is in original.pdf
                echo file_get_contents("../../images/" . get_wiki_file_path($filename));
            }
        }
    }

}

?>
