<?php
/**
 * Copyright 2013 Drexel University
 */

// SPECIAL PAGE: This page is shown when you click on a file that is stored on IRODS.
//				 Information about the specified file is shown, like metadata and AVUs
// Feature: add/edit/delete the AVUs
// TO DO: implement user permissions.

class SpecialRodsWiki extends SpecialPage {

    private $dir;

    function __construct() {
        $this->dir = dirname(__FILE__) . '/';

        require_once ($this->dir . "RodsWiki.functions.php");

        parent::__construct('IrodsMetadataPage');
    }

    function execute($par) {
        global $wgOut, $wgScriptPath, $wgUser,
        $RodsWikiSocial, $RodsWikiTriggerRules;

        require_once ($this->dir . "RodsWiki.functions.php");
        $this->setHeaders();

        $output = '<script src="' . $wgScriptPath . '/extensions/RodsWiki/SpecialMetadata.js"></script>';
        $output .= '<link href="' . $wgScriptPath . '/extensions/RodsWiki/SpecialMetadata.css" rel="stylesheet">';
        $wgOut->addHTML($output);

        if (isset($_GET['filename'])) {

            $full_file_path = $filename = $format = null;
            get_irods_path_info($_GET['filename'], $full_file_path, $filename, $format);

            $wgOut->setPagetitle(basename($filename));

            if (isset($_GET['ticket']))
                $IrodsUser = getAccount(false, $_GET['ticket']);
            else
                $IrodsUser = getAccount();

            $rights = get_user_rights_on_irods_file($full_file_path, $IrodsUser->user);

            $create_ticket = $modify = $read = false;
            $useraccess = 1;
            if ($rights->COL_DATA_ACCESS_NAME == "own")
                $create_ticket = true;
            elseif ($rights->COL_DATA_ACCESS_NAME == "modify object")
                $modify = true;
            elseif ($rights->COL_DATA_ACCESS_NAME == "read object")
                $read = true;
            else
                $useraccess = 0;

            if (existFile($full_file_path)) { // checks if the file exists on irods
                // Display the page content
                $metadata = getIrodsMetadata($full_file_path);

                // Execute different tasks
                // check for user rights
                if ($wgUser->mId > 0) { // checks if there is a user logged in
                    if (isset($_POST['avu_delete'])) {
                        deleteAVU($full_file_path, $_POST['avu_attr'], $_POST['avu_val'], $_POST['avu_unit']);
                        die('DELETE AVU - OK (' . $_POST['avu_attr'] . ' - ' . $_POST['avu_val'] . ' - ' . $_POST['avu_unit'] . ')');
                    }
                    if (isset($_POST['avu_add'])) {
                        addAVU($full_file_path, $_POST['avu_attr'], $_POST['avu_val'], $_POST['avu_unit']);
                        die('ADD AVU OK (' . $_POST['avu_attr'] . ' - ' . $_POST['avu_val'] . ' - ' . $_POST['avu_unit'] . ')');
                    }
                    if (isset($_POST['get_full_path'])) {
                        $file_info = RodsWikiParsers::RodsWikiRenderParserFunction(null, $_POST['get_full_path'], null);
                        die(json_encode(array($file_info['no_files'], $file_info['file_path'])));
                    }
                }

                // Loat linked files now, since it is user to display the thumbnail
                global $wgScriptPath;
                $array_links_to_files = null;
                $linked_files_output = SpecialRodsWiki::search_linked_files($full_file_path, $array_links_to_files);
                $key = array_search('thumbnail_of', $array_links_to_files['COL_META_DATA_ATTR_NAME']);

                $thumbnail = '';
                if ($key) {
                    if (strtolower($array_links_to_files['COL_META_DATA_ATTR_UNITS'][$key]) == 'irods path') {
                        if (isset($_GET['ticket']))
                            $ticket = "&ticket=" . $_GET['ticket'];
                        else
                            $ticket = '';
                        $thumbnail = "<img style=\"border: 1px solid black\" src=\"" . $wgScriptPath . '/index.php/Special:SpecialIrodsFileDownload?filename=' . $array_links_to_files['COL_COLL_NAME'][$key] . "/" . $array_links_to_files['COL_DATA_NAME'][$key] . $ticket . "\" align=\"right\"><br />";
                    }
                }

                $metadata = getIrodsMetadata($full_file_path);

                // Add wikitext header for the metadata
                $wgOut->addWikiText("== " . $filename . " ==");

                $wgOut->addHTML($thumbnail);
                $wgOut->addHTML(SpecialRodsWiki::print_file_system_info($full_file_path, $metadata));

                // Download button
                $ticket = '';
                if ($create_ticket || $modify || $read) {
                    if (isset($_GET['ticket']))
                        $ticket = '&ticket=' . $_GET['ticket'];
                    $wgOut->addHTML('<button class="icon" value="Download" id="btnDownloadFile" link="' . $wgScriptPath . '/index.php/Special:SpecialIrodsFileDownload?filename=' . $_GET['filename'] . $ticket . '"><span class="icon_download">&nbsp;</span>&nbsp;<span class="btn_caption">Download ' . $filename . '</span></button>');
                }



                // Add wikitext header for the metadata
                $wgOut->addWikiText("== Metadata ==");



                if ($wgUser->mId > 0) // checks if there is a user logged in
                    $wgOut->addHTML(SpecialRodsWiki::print_irods_metadata($metadata, $useraccess));
                else // in case there is no logged in visitor:
                    $wgOut->addHTML(SpecialRodsWiki::print_irods_metadata($metadata, 0));

                // Display the files that have links to the accessed one
                if ($linked_files_output != FALSE) {
                    $wgOut->addHTML('<br />');
                    $wgOut->addWikiText("== Files having links to $filename ==");
                    $wgOut->addHTML($linked_files_output);
                }

                // Add markup
                $wgOut->addHTML('<br />');
                $wgOut->addWikiText("== File Integration in MediaWiki ==");
                $files_with_this_alias = get_files_based_on_alias($filename);

                $tag_content = $full_file_path;
                if ($files_with_this_alias != false)
                    if (sizeof($files_with_this_alias['COL_DATA_NAME']) == 1)
                        $tag_content = $filename;
                    else {
                        $add_second_tag = true;
                    }


                $wgOut->addHTML('<pre>{{#irods:' . $tag_content . '}}</pre>'); // For file link
                if ($add_second_tag)
                    $wgOut->addHTML('<pre>{{#irods:' . $filename . '}} <font color="blue">   // this displays all the files having the alias: `' . $filename . '`</font></pre><br />'); // For file link



                    
                // Add Rules section
                if (isset($RodsWikiTriggerRules))
                    if ($RodsWikiTriggerRules && $wgUser->mId > 0) {
                        $wgOut->addHTML('<br />');
                        $wgOut->addWikiText('((TRIGGERRULES))' . json_encode(array_merge($metadata, array('full_path' => $full_file_path))));
                    }


                // Add Social section
                if (isset($RodsWikiSocial))
                    if ($RodsWikiSocial) {
                        $wgOut->addHTML('<br /><br />');
                        $wgOut->addWikiText('((SOCIALNETWORKBLOCK))' . json_encode($metadata));
                    }
            } else {
                $wgOut->setPagetitle("Missing file " . $filename);
                $wgOut->addHTML('We are sorry but the file ' . $filename . ' was not found. <a href="' . $wgScriptPath . '/index.php/Special:SpecialIrodsUploadPage?filename=' . $filename . ((isset($_GET['redir']) ? ("&redir=" . $_GET['redir']) : '')) . '">Upload this file!</a>');
            }
        } else { // in case there is no given filename
            $wgOut->setPagetitle("Filename error");
            $wgOut->addHTML('The failed to specify a filename.');
        }

        $wgOut->addHTML('<script src="' . $wgScriptPath . '/extensions/RodsWiki/SpecialMetadataEvents.js"></script>');
    }

    function print_file_system_info(&$full_file_path, &$irods_metadata) {

        return "
            <table cellpadding=\"4\" cellspacing=\"4\" border=0>
                <tr align=\"left\"> <td> File size </td> <td> " . get_formated_size($irods_metadata['size']) . " </td> </tr>
                <tr align=\"left\"> <td> Created </td> <td> " . date("F j, Y, g:i a", $irods_metadata['ctime']) . " </td> </tr>
                <tr align=\"left\"> <td> Modified </td> <td> " . date("F j, Y, g:i a", $irods_metadata['mtime']) . " </td> </tr>
                <tr align=\"left\"> <td> System full path: </td> <td> " . $full_file_path . " </td> </tr>
            </table>";
    }

    function print_irods_metadata(&$irods_metadata, $useraccess = 0) { // this function is like a template, which shows the metadata
        global $configIrodsRootPath, $configFFUrl, $configFFUnit, $configFFAttr, $configAttrAliases, $configUnitAliases;

        // create the output list of data
        $metadata = $irods_metadata['AVUs'];
        $output .= "<table cellpadding=\"4\" cellspacing=\"4\" border=\"0\" id=\"avu_table\">\n";
        $output .= "<tr align=\"left\"> <th width=\"150\"> Attribute </th> <th width=\"150\"> Value </th> <th width=\"150\"> Units </th> <th> </th> </tr>\n";
        $output .= "</table>";
        $output .= "<script> var configFFUrl = \"" . $configFFUrl . "\"; var configFFAttr = \"" . $configFFAttr . "\";";
        $output .= 
            "var attrAliases = " . json_encode($configAttrAliases) .";" .
            "var unitAliases = " . json_encode($configUnitAliases) .";" ; 



        if ($useraccess == 0) { // this is for case when the user should not have access to edit/add irods data
            for ($i = 0; $i < sizeof($metadata); $i++) {
                $output .= "add_row('" . $metadata[$i][0] . "', '" . $metadata[$i][1] . "', '" . $metadata[$i][2] . "', false, '" . $metadata[$i][1] . "', '" . $link . "', false);";
            }
        } else if ($useraccess == 1) {

            for ($i = 0; $i < sizeof($metadata); $i++) {
                if ($metadata[$i][0] == $configFFAttr && $metadata[$i][2] == "http://www.w3.org/2001/XMLSchema#anyURI") {
                    $output .= "add_row('" . $metadata[$i][0] . "', '" . $metadata[$i][1] . "', '" . $metadata[$i][2] . "', false, '" . $metadata[$i][1] . "', '" . $configFFUrl . 'edit?entity=' . $metadata[$i][1] . "', true);";
                } elseif ($metadata[$i][0] == "wiki_alias") {
                    $output .= "add_row('" . $metadata[$i][0] . "', '" . $metadata[$i][1] . "', '" . $metadata[$i][2] . "', false, '" . $metadata[$i][1] . "', '', false);";
                }
                else
                    $output .= "add_row('" . $metadata[$i][0] . "', '" . $metadata[$i][1] . "', '" . $metadata[$i][2] . "', false);";
            }
        }
        $output .= "</script>";

        if ($useraccess == 1) {

            // AVU: MediaWiki Alias
            $wiki_alias_pos = -1;
            for ($i = 0; $i < sizeof($metadata); $i++)
                if ($metadata[$i][0] == "wiki_alias")
                    $wiki_alias_pos = $i;

            $mediaWikiAliasAddHidden = $mediaWikiAliasRemoveHidden = " style=\"display:none\"";

            if ($wiki_alias_pos >= 0)
                $mediaWikiAliasRemoveHidden = "";
            else
                $mediaWikiAliasAddHidden = "";

            $output .= "<table>";
            $output .= "<tr><td>";
            $output .= "<button class=\"icon\" onclick=\"add_row('', '', '', true);\" value=\"Add metadata\"><span class=\"icon_add\">&nbsp;</span>&nbsp;<span class=\"btn_caption\">Add metadata</span></button>";
            $output .= "<button class=\"icon\" " . $mediaWikiAliasRemoveHidden . " start=\"" . ($wiki_alias_pos * 3) . "\" id=\"btnMediaWikiRemoveAlias\" onclick=\"remove_wiki_alias(" . ($wiki_alias_pos * 3) . ");\" value=\"Wiki Alias\"><span class=\"icon_delete\">&nbsp;</span>&nbsp;<span class=\"btn_caption\">Remove Alias</span></button>";
            $output .= "<button class=\"icon\" " . $mediaWikiAliasAddHidden . " start=\"" . ($wiki_alias_pos * 3) . "\" id=\"btnMediaWikiAddAlias\" onclick=\"add_wiki_alias('wiki_alias', '" . $irods_metadata['filename'] . "', '', true);\" value=\"Wiki Alias\"><span class=\"icon_add\">&nbsp;</span>&nbsp;<span class=\"btn_caption\">Add Alias</span></button>";
            $output .= "<button class=\"icon\" id=\"btnShowLinkToFileTable\" value=\"Link to file\"><span class=\"icon_add\">&nbsp;</span>&nbsp;<span class=\"btn_caption\">Add link to file</span></button>";
            $output .= "</td></tr>";
            $output .= "</table>";

            $output .= "<br>";
            $output .= "<table style=\"display: none\" id=\"tbl_link_file\"><tr> <td colspan = 2> <i><b>Link to a file</b></i> </td>  <td>  </td> </tr>\n";
            $output .= "<tr> <td> Link&nbsp;name: </td> <td> <input type=\"text\" id=\"link_attribute\" value=\"link_to_file\" size=\"40\"> </td> <td>  </td> </tr>\n";
            $output .= "<tr> <td> File&nbsp;path: </td> <td> <input type=\"text\" id=\"link_to_file\" id=\"link_to_file\" size=\"40\"> </td> <td>  </td> </tr>\n";
            $output .= "<tr> <td></td> <td colspan=\"3\">Please use one of the following formats:";
            $output .= "<li>iRODS Path (e.g. <b>" . $configIrodsRootPath . "<i>image.jpg</i></b>)</li />";
            $output .= "<li>MediaWiki iRODS File URL (e.g. <b>http://" . $_SERVER["HTTP_HOST"] . dirname($_SERVER["REQUEST_URI"]) . "/index.php?title=Special:IrodsMetadataPage&filename=<i>image.jpg</i></b> )</li>\n";
            $output .= "<li>Filename (e.g. <b><i>Image.jpg</i></b>)</li></td> <td>  </td> </tr>\n";
            $output .= "<tr> <td colspan=\"2\"><button class=\"icon\" value=\"Link to file\" id=\"btnSaveLinkToFile\"><span class=\"icon_add\">&nbsp;</span>&nbsp;<span class=\"btn_caption\">Save link to file</span></button><button class=\"icon\" value=\"Cancel\" id=\"btnCancelLinkToFile\"><span class=\"icon_cancel\">&nbsp;</span>&nbsp;<span class=\"btn_caption\">Cancel</span></button></td></tr>";
            $output .= "</table>";
        }

        return $output;
    }

    function change_metadata($filepath, &$new_raw_data) { // this function handles any change that was made (AVUs added/edited/deleted)
        $no_fields = (sizeof($new_raw_data) - 3) / 6;
        $edit_fields = array();
        $delete_fields = array();

        // create the task lists
        for ($i = 1; $i <= $no_fields; $i++) {
            if (trim($new_raw_data['avuattrnew' . $i]) == "" || trim($new_raw_data['avuvalnew' . $i]) == "")
                $delete_fields[] = array(trim($new_raw_data['avuattrold' . $i]), trim($new_raw_data['avuvalold' . $i]), trim($new_raw_data['avuunitold' . $i]));
            else
                $edit_fields[] = array(trim($new_raw_data['avuattrold' . $i]), trim($new_raw_data['avuvalold' . $i]), trim($new_raw_data['avuunitold' . $i]), trim($new_raw_data['avuattrnew' . $i]), trim($new_raw_data['avuvalnew' . $i]), trim($new_raw_data['avuunitnew' . $i]));
        }

        // editing the fields
        for ($i = 0; $i < sizeof($edit_fields); $i++) {
            updateAVUs($filepath, $edit_fields[$i][0], $edit_fields[$i][1], $edit_fields[$i][2], $edit_fields[$i][3], $edit_fields[$i][4], $edit_fields[$i][5]);
        }

        // deleting AVUs
        for ($i = 0; $i < sizeof($delete_fields); $i++) {
            deleteAVU($filepath, $delete_fields[$i][0], $delete_fields[$i][1], $delete_fields[$i][2]);
        }

        // adding AVUs
        if (trim($new_raw_data['avuaddattr']) != "" && trim($new_raw_data['avuaddval']) != "")
            addAVU($filepath, trim($new_raw_data['avuaddattr']), trim($new_raw_data['avuaddval']), trim($new_raw_data['avuaddunit']));
    }

    function search_linked_files($original_file_full_path, &$result_values) {
        global $wgScriptPath, $configAttrAliases;

        $IrodsAccount = getAccount();
        // look for the mediaWiki Alias
        $flds = array("COL_DATA_NAME" => NULL, "COL_DATA_SIZE" => NULL,
            "COL_COLL_NAME" => NULL, 'COL_META_DATA_ATTR_NAME' => NULL,
            'COL_META_DATA_ATTR_UNITS' => NULL
        );
        $select = new RODSGenQueSelFlds(array_keys($flds), array_values($flds));
        $condition = new RODSGenQueConds();
        $condition->add('COL_META_DATA_ATTR_NAME', 'LIKE', '%%');
        $condition->add('COL_META_DATA_ATTR_VALUE', 'LIKE', $original_file_full_path);
        $condition->add('COL_COLL_NAME', 'not like', '%/trash/%');
        $condition->add('COL_COLL_NAME', 'like', '/%'); // recursive search

        $conn = RODSConnManager::getConn($IrodsAccount);
        $results = $conn->query($select, $condition, 0, -1);
        RODSConnManager::releaseConn($conn);

        $result_values = $results->getValues();

        if (sizeof($result_values['COL_DATA_NAME']) == 0)
            return false;

        function get_different_path_file2($file1, $file2) {
            $q = 0;
            for ($i = 0; $i < min(strlen($file1), strlen($file2)); $i++) {
                if ($file1[$i] == $file2[$i])
                    $q++;
                else
                    break;
            }

            return substr($file2, $q);
        }

        $output = '<table cellpadding="4" cellspacing="4" border=0>';
        $output .= "<tr><th>File</th><th>From Attribute</th></tr>";
        for ($i = 0; $i < sizeof($result_values["COL_DATA_NAME"]); $i++) {
            $col = get_different_path_file2($original_file_full_path, $result_values["COL_COLL_NAME"][$i]);
            if (substr($col, 0, 5) == "arch-")
                $col = substr($col, 1 + strpos($col, '/'));
            $attr_val = $result_values["COL_META_DATA_ATTR_NAME"][$i];
            if (isset($configAttrAliases[$attr_val]))
                $attr_alias = '<abbr title="' . $attr_val . '">' . $configAttrAliases[$attr_val] . '</attr>';
            else
                $attr_alias = $attr_val;
            $col .= '/';
            $output .= "<tr>";
            $output .= "<td><a href=\"" . $wgScriptPath . '/index.php/Special:IrodsMetadataPage?filename=' . $result_values["COL_COLL_NAME"][$i] . '/' . $result_values["COL_DATA_NAME"][$i] . '">' . $col . $result_values["COL_DATA_NAME"][$i] . "</a></td><td>" . $attr_alias . "</td>";
            $output .= "</tr>";
        }
        $output .= "</table>";


        return $output;
    }

}

?>
